// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#[allow(unused_imports)]
use crate::auto::*;

#[doc(alias = "PangoGlyph")]
pub type Glyph = u32;
#[doc(alias = "PangoGlyphUnit")]
pub type GlyphUnit = i32;
#[doc(alias = "PangoLayoutRun")]
pub type LayoutRun = GlyphItem;
