// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GActionEntry", sizeof(GActionEntry), alignof(GActionEntry));
    printf("%s;%zu;%zu\n", "GActionGroupInterface", sizeof(GActionGroupInterface), alignof(GActionGroupInterface));
    printf("%s;%zu;%zu\n", "GActionInterface", sizeof(GActionInterface), alignof(GActionInterface));
    printf("%s;%zu;%zu\n", "GActionMapInterface", sizeof(GActionMapInterface), alignof(GActionMapInterface));
    printf("%s;%zu;%zu\n", "GAppInfoCreateFlags", sizeof(GAppInfoCreateFlags), alignof(GAppInfoCreateFlags));
    printf("%s;%zu;%zu\n", "GAppInfoIface", sizeof(GAppInfoIface), alignof(GAppInfoIface));
    printf("%s;%zu;%zu\n", "GAppLaunchContext", sizeof(GAppLaunchContext), alignof(GAppLaunchContext));
    printf("%s;%zu;%zu\n", "GAppLaunchContextClass", sizeof(GAppLaunchContextClass), alignof(GAppLaunchContextClass));
    printf("%s;%zu;%zu\n", "GApplication", sizeof(GApplication), alignof(GApplication));
    printf("%s;%zu;%zu\n", "GApplicationClass", sizeof(GApplicationClass), alignof(GApplicationClass));
    printf("%s;%zu;%zu\n", "GApplicationCommandLine", sizeof(GApplicationCommandLine), alignof(GApplicationCommandLine));
    printf("%s;%zu;%zu\n", "GApplicationCommandLineClass", sizeof(GApplicationCommandLineClass), alignof(GApplicationCommandLineClass));
    printf("%s;%zu;%zu\n", "GApplicationFlags", sizeof(GApplicationFlags), alignof(GApplicationFlags));
    printf("%s;%zu;%zu\n", "GAskPasswordFlags", sizeof(GAskPasswordFlags), alignof(GAskPasswordFlags));
    printf("%s;%zu;%zu\n", "GAsyncInitableIface", sizeof(GAsyncInitableIface), alignof(GAsyncInitableIface));
    printf("%s;%zu;%zu\n", "GAsyncResultIface", sizeof(GAsyncResultIface), alignof(GAsyncResultIface));
    printf("%s;%zu;%zu\n", "GBufferedInputStream", sizeof(GBufferedInputStream), alignof(GBufferedInputStream));
    printf("%s;%zu;%zu\n", "GBufferedInputStreamClass", sizeof(GBufferedInputStreamClass), alignof(GBufferedInputStreamClass));
    printf("%s;%zu;%zu\n", "GBufferedOutputStream", sizeof(GBufferedOutputStream), alignof(GBufferedOutputStream));
    printf("%s;%zu;%zu\n", "GBufferedOutputStreamClass", sizeof(GBufferedOutputStreamClass), alignof(GBufferedOutputStreamClass));
    printf("%s;%zu;%zu\n", "GBusNameOwnerFlags", sizeof(GBusNameOwnerFlags), alignof(GBusNameOwnerFlags));
    printf("%s;%zu;%zu\n", "GBusNameWatcherFlags", sizeof(GBusNameWatcherFlags), alignof(GBusNameWatcherFlags));
    printf("%s;%zu;%zu\n", "GBusType", sizeof(GBusType), alignof(GBusType));
    printf("%s;%zu;%zu\n", "GCancellable", sizeof(GCancellable), alignof(GCancellable));
    printf("%s;%zu;%zu\n", "GCancellableClass", sizeof(GCancellableClass), alignof(GCancellableClass));
    printf("%s;%zu;%zu\n", "GCharsetConverterClass", sizeof(GCharsetConverterClass), alignof(GCharsetConverterClass));
    printf("%s;%zu;%zu\n", "GConverterFlags", sizeof(GConverterFlags), alignof(GConverterFlags));
    printf("%s;%zu;%zu\n", "GConverterIface", sizeof(GConverterIface), alignof(GConverterIface));
    printf("%s;%zu;%zu\n", "GConverterInputStream", sizeof(GConverterInputStream), alignof(GConverterInputStream));
    printf("%s;%zu;%zu\n", "GConverterInputStreamClass", sizeof(GConverterInputStreamClass), alignof(GConverterInputStreamClass));
    printf("%s;%zu;%zu\n", "GConverterOutputStream", sizeof(GConverterOutputStream), alignof(GConverterOutputStream));
    printf("%s;%zu;%zu\n", "GConverterOutputStreamClass", sizeof(GConverterOutputStreamClass), alignof(GConverterOutputStreamClass));
    printf("%s;%zu;%zu\n", "GConverterResult", sizeof(GConverterResult), alignof(GConverterResult));
    printf("%s;%zu;%zu\n", "GCredentialsType", sizeof(GCredentialsType), alignof(GCredentialsType));
    printf("%s;%zu;%zu\n", "GDBusAnnotationInfo", sizeof(GDBusAnnotationInfo), alignof(GDBusAnnotationInfo));
    printf("%s;%zu;%zu\n", "GDBusArgInfo", sizeof(GDBusArgInfo), alignof(GDBusArgInfo));
    printf("%s;%zu;%zu\n", "GDBusCallFlags", sizeof(GDBusCallFlags), alignof(GDBusCallFlags));
    printf("%s;%zu;%zu\n", "GDBusCapabilityFlags", sizeof(GDBusCapabilityFlags), alignof(GDBusCapabilityFlags));
    printf("%s;%zu;%zu\n", "GDBusConnectionFlags", sizeof(GDBusConnectionFlags), alignof(GDBusConnectionFlags));
    printf("%s;%zu;%zu\n", "GDBusError", sizeof(GDBusError), alignof(GDBusError));
    printf("%s;%zu;%zu\n", "GDBusErrorEntry", sizeof(GDBusErrorEntry), alignof(GDBusErrorEntry));
    printf("%s;%zu;%zu\n", "GDBusInterfaceIface", sizeof(GDBusInterfaceIface), alignof(GDBusInterfaceIface));
    printf("%s;%zu;%zu\n", "GDBusInterfaceInfo", sizeof(GDBusInterfaceInfo), alignof(GDBusInterfaceInfo));
    printf("%s;%zu;%zu\n", "GDBusInterfaceSkeleton", sizeof(GDBusInterfaceSkeleton), alignof(GDBusInterfaceSkeleton));
    printf("%s;%zu;%zu\n", "GDBusInterfaceSkeletonClass", sizeof(GDBusInterfaceSkeletonClass), alignof(GDBusInterfaceSkeletonClass));
    printf("%s;%zu;%zu\n", "GDBusInterfaceSkeletonFlags", sizeof(GDBusInterfaceSkeletonFlags), alignof(GDBusInterfaceSkeletonFlags));
    printf("%s;%zu;%zu\n", "GDBusInterfaceVTable", sizeof(GDBusInterfaceVTable), alignof(GDBusInterfaceVTable));
    printf("%s;%zu;%zu\n", "GDBusMessageByteOrder", sizeof(GDBusMessageByteOrder), alignof(GDBusMessageByteOrder));
    printf("%s;%zu;%zu\n", "GDBusMessageFlags", sizeof(GDBusMessageFlags), alignof(GDBusMessageFlags));
    printf("%s;%zu;%zu\n", "GDBusMessageHeaderField", sizeof(GDBusMessageHeaderField), alignof(GDBusMessageHeaderField));
    printf("%s;%zu;%zu\n", "GDBusMessageType", sizeof(GDBusMessageType), alignof(GDBusMessageType));
    printf("%s;%zu;%zu\n", "GDBusMethodInfo", sizeof(GDBusMethodInfo), alignof(GDBusMethodInfo));
    printf("%s;%zu;%zu\n", "GDBusNodeInfo", sizeof(GDBusNodeInfo), alignof(GDBusNodeInfo));
    printf("%s;%zu;%zu\n", "GDBusObjectIface", sizeof(GDBusObjectIface), alignof(GDBusObjectIface));
    printf("%s;%zu;%zu\n", "GDBusObjectManagerClient", sizeof(GDBusObjectManagerClient), alignof(GDBusObjectManagerClient));
    printf("%s;%zu;%zu\n", "GDBusObjectManagerClientClass", sizeof(GDBusObjectManagerClientClass), alignof(GDBusObjectManagerClientClass));
    printf("%s;%zu;%zu\n", "GDBusObjectManagerClientFlags", sizeof(GDBusObjectManagerClientFlags), alignof(GDBusObjectManagerClientFlags));
    printf("%s;%zu;%zu\n", "GDBusObjectManagerIface", sizeof(GDBusObjectManagerIface), alignof(GDBusObjectManagerIface));
    printf("%s;%zu;%zu\n", "GDBusObjectManagerServer", sizeof(GDBusObjectManagerServer), alignof(GDBusObjectManagerServer));
    printf("%s;%zu;%zu\n", "GDBusObjectManagerServerClass", sizeof(GDBusObjectManagerServerClass), alignof(GDBusObjectManagerServerClass));
    printf("%s;%zu;%zu\n", "GDBusObjectProxy", sizeof(GDBusObjectProxy), alignof(GDBusObjectProxy));
    printf("%s;%zu;%zu\n", "GDBusObjectProxyClass", sizeof(GDBusObjectProxyClass), alignof(GDBusObjectProxyClass));
    printf("%s;%zu;%zu\n", "GDBusObjectSkeleton", sizeof(GDBusObjectSkeleton), alignof(GDBusObjectSkeleton));
    printf("%s;%zu;%zu\n", "GDBusObjectSkeletonClass", sizeof(GDBusObjectSkeletonClass), alignof(GDBusObjectSkeletonClass));
    printf("%s;%zu;%zu\n", "GDBusPropertyInfo", sizeof(GDBusPropertyInfo), alignof(GDBusPropertyInfo));
    printf("%s;%zu;%zu\n", "GDBusPropertyInfoFlags", sizeof(GDBusPropertyInfoFlags), alignof(GDBusPropertyInfoFlags));
    printf("%s;%zu;%zu\n", "GDBusProxy", sizeof(GDBusProxy), alignof(GDBusProxy));
    printf("%s;%zu;%zu\n", "GDBusProxyClass", sizeof(GDBusProxyClass), alignof(GDBusProxyClass));
    printf("%s;%zu;%zu\n", "GDBusProxyFlags", sizeof(GDBusProxyFlags), alignof(GDBusProxyFlags));
    printf("%s;%zu;%zu\n", "GDBusSendMessageFlags", sizeof(GDBusSendMessageFlags), alignof(GDBusSendMessageFlags));
    printf("%s;%zu;%zu\n", "GDBusServerFlags", sizeof(GDBusServerFlags), alignof(GDBusServerFlags));
    printf("%s;%zu;%zu\n", "GDBusSignalFlags", sizeof(GDBusSignalFlags), alignof(GDBusSignalFlags));
    printf("%s;%zu;%zu\n", "GDBusSignalInfo", sizeof(GDBusSignalInfo), alignof(GDBusSignalInfo));
    printf("%s;%zu;%zu\n", "GDBusSubtreeFlags", sizeof(GDBusSubtreeFlags), alignof(GDBusSubtreeFlags));
    printf("%s;%zu;%zu\n", "GDBusSubtreeVTable", sizeof(GDBusSubtreeVTable), alignof(GDBusSubtreeVTable));
    printf("%s;%zu;%zu\n", "GDataInputStream", sizeof(GDataInputStream), alignof(GDataInputStream));
    printf("%s;%zu;%zu\n", "GDataInputStreamClass", sizeof(GDataInputStreamClass), alignof(GDataInputStreamClass));
    printf("%s;%zu;%zu\n", "GDataOutputStream", sizeof(GDataOutputStream), alignof(GDataOutputStream));
    printf("%s;%zu;%zu\n", "GDataOutputStreamClass", sizeof(GDataOutputStreamClass), alignof(GDataOutputStreamClass));
    printf("%s;%zu;%zu\n", "GDataStreamByteOrder", sizeof(GDataStreamByteOrder), alignof(GDataStreamByteOrder));
    printf("%s;%zu;%zu\n", "GDataStreamNewlineType", sizeof(GDataStreamNewlineType), alignof(GDataStreamNewlineType));
    printf("%s;%zu;%zu\n", "GDatagramBasedInterface", sizeof(GDatagramBasedInterface), alignof(GDatagramBasedInterface));
    printf("%s;%zu;%zu\n", "GDebugControllerDBus", sizeof(GDebugControllerDBus), alignof(GDebugControllerDBus));
    printf("%s;%zu;%zu\n", "GDebugControllerDBusClass", sizeof(GDebugControllerDBusClass), alignof(GDebugControllerDBusClass));
    printf("%s;%zu;%zu\n", "GDebugControllerInterface", sizeof(GDebugControllerInterface), alignof(GDebugControllerInterface));
    printf("%s;%zu;%zu\n", "GDesktopAppInfoClass", sizeof(GDesktopAppInfoClass), alignof(GDesktopAppInfoClass));
    printf("%s;%zu;%zu\n", "GDesktopAppInfoLookupIface", sizeof(GDesktopAppInfoLookupIface), alignof(GDesktopAppInfoLookupIface));
    printf("%s;%zu;%zu\n", "GDriveIface", sizeof(GDriveIface), alignof(GDriveIface));
    printf("%s;%zu;%zu\n", "GDriveStartFlags", sizeof(GDriveStartFlags), alignof(GDriveStartFlags));
    printf("%s;%zu;%zu\n", "GDriveStartStopType", sizeof(GDriveStartStopType), alignof(GDriveStartStopType));
    printf("%s;%zu;%zu\n", "GDtlsClientConnectionInterface", sizeof(GDtlsClientConnectionInterface), alignof(GDtlsClientConnectionInterface));
    printf("%s;%zu;%zu\n", "GDtlsConnectionInterface", sizeof(GDtlsConnectionInterface), alignof(GDtlsConnectionInterface));
    printf("%s;%zu;%zu\n", "GDtlsServerConnectionInterface", sizeof(GDtlsServerConnectionInterface), alignof(GDtlsServerConnectionInterface));
    printf("%s;%zu;%zu\n", "GEmblemOrigin", sizeof(GEmblemOrigin), alignof(GEmblemOrigin));
    printf("%s;%zu;%zu\n", "GEmblemedIcon", sizeof(GEmblemedIcon), alignof(GEmblemedIcon));
    printf("%s;%zu;%zu\n", "GEmblemedIconClass", sizeof(GEmblemedIconClass), alignof(GEmblemedIconClass));
    printf("%s;%zu;%zu\n", "GFileAttributeInfo", sizeof(GFileAttributeInfo), alignof(GFileAttributeInfo));
    printf("%s;%zu;%zu\n", "GFileAttributeInfoFlags", sizeof(GFileAttributeInfoFlags), alignof(GFileAttributeInfoFlags));
    printf("%s;%zu;%zu\n", "GFileAttributeInfoList", sizeof(GFileAttributeInfoList), alignof(GFileAttributeInfoList));
    printf("%s;%zu;%zu\n", "GFileAttributeStatus", sizeof(GFileAttributeStatus), alignof(GFileAttributeStatus));
    printf("%s;%zu;%zu\n", "GFileAttributeType", sizeof(GFileAttributeType), alignof(GFileAttributeType));
    printf("%s;%zu;%zu\n", "GFileCopyFlags", sizeof(GFileCopyFlags), alignof(GFileCopyFlags));
    printf("%s;%zu;%zu\n", "GFileCreateFlags", sizeof(GFileCreateFlags), alignof(GFileCreateFlags));
    printf("%s;%zu;%zu\n", "GFileDescriptorBasedIface", sizeof(GFileDescriptorBasedIface), alignof(GFileDescriptorBasedIface));
    printf("%s;%zu;%zu\n", "GFileEnumerator", sizeof(GFileEnumerator), alignof(GFileEnumerator));
    printf("%s;%zu;%zu\n", "GFileEnumeratorClass", sizeof(GFileEnumeratorClass), alignof(GFileEnumeratorClass));
    printf("%s;%zu;%zu\n", "GFileIOStream", sizeof(GFileIOStream), alignof(GFileIOStream));
    printf("%s;%zu;%zu\n", "GFileIOStreamClass", sizeof(GFileIOStreamClass), alignof(GFileIOStreamClass));
    printf("%s;%zu;%zu\n", "GFileIface", sizeof(GFileIface), alignof(GFileIface));
    printf("%s;%zu;%zu\n", "GFileInputStream", sizeof(GFileInputStream), alignof(GFileInputStream));
    printf("%s;%zu;%zu\n", "GFileInputStreamClass", sizeof(GFileInputStreamClass), alignof(GFileInputStreamClass));
    printf("%s;%zu;%zu\n", "GFileMeasureFlags", sizeof(GFileMeasureFlags), alignof(GFileMeasureFlags));
    printf("%s;%zu;%zu\n", "GFileMonitor", sizeof(GFileMonitor), alignof(GFileMonitor));
    printf("%s;%zu;%zu\n", "GFileMonitorClass", sizeof(GFileMonitorClass), alignof(GFileMonitorClass));
    printf("%s;%zu;%zu\n", "GFileMonitorEvent", sizeof(GFileMonitorEvent), alignof(GFileMonitorEvent));
    printf("%s;%zu;%zu\n", "GFileMonitorFlags", sizeof(GFileMonitorFlags), alignof(GFileMonitorFlags));
    printf("%s;%zu;%zu\n", "GFileOutputStream", sizeof(GFileOutputStream), alignof(GFileOutputStream));
    printf("%s;%zu;%zu\n", "GFileOutputStreamClass", sizeof(GFileOutputStreamClass), alignof(GFileOutputStreamClass));
    printf("%s;%zu;%zu\n", "GFileQueryInfoFlags", sizeof(GFileQueryInfoFlags), alignof(GFileQueryInfoFlags));
    printf("%s;%zu;%zu\n", "GFileType", sizeof(GFileType), alignof(GFileType));
    printf("%s;%zu;%zu\n", "GFilenameCompleterClass", sizeof(GFilenameCompleterClass), alignof(GFilenameCompleterClass));
    printf("%s;%zu;%zu\n", "GFilesystemPreviewType", sizeof(GFilesystemPreviewType), alignof(GFilesystemPreviewType));
    printf("%s;%zu;%zu\n", "GFilterInputStream", sizeof(GFilterInputStream), alignof(GFilterInputStream));
    printf("%s;%zu;%zu\n", "GFilterInputStreamClass", sizeof(GFilterInputStreamClass), alignof(GFilterInputStreamClass));
    printf("%s;%zu;%zu\n", "GFilterOutputStream", sizeof(GFilterOutputStream), alignof(GFilterOutputStream));
    printf("%s;%zu;%zu\n", "GFilterOutputStreamClass", sizeof(GFilterOutputStreamClass), alignof(GFilterOutputStreamClass));
    printf("%s;%zu;%zu\n", "GIOErrorEnum", sizeof(GIOErrorEnum), alignof(GIOErrorEnum));
    printf("%s;%zu;%zu\n", "GIOModuleScopeFlags", sizeof(GIOModuleScopeFlags), alignof(GIOModuleScopeFlags));
    printf("%s;%zu;%zu\n", "GIOStream", sizeof(GIOStream), alignof(GIOStream));
    printf("%s;%zu;%zu\n", "GIOStreamClass", sizeof(GIOStreamClass), alignof(GIOStreamClass));
    printf("%s;%zu;%zu\n", "GIOStreamSpliceFlags", sizeof(GIOStreamSpliceFlags), alignof(GIOStreamSpliceFlags));
    printf("%s;%zu;%zu\n", "GIconIface", sizeof(GIconIface), alignof(GIconIface));
    printf("%s;%zu;%zu\n", "GInetAddress", sizeof(GInetAddress), alignof(GInetAddress));
    printf("%s;%zu;%zu\n", "GInetAddressClass", sizeof(GInetAddressClass), alignof(GInetAddressClass));
    printf("%s;%zu;%zu\n", "GInetAddressMask", sizeof(GInetAddressMask), alignof(GInetAddressMask));
    printf("%s;%zu;%zu\n", "GInetAddressMaskClass", sizeof(GInetAddressMaskClass), alignof(GInetAddressMaskClass));
    printf("%s;%zu;%zu\n", "GInetSocketAddress", sizeof(GInetSocketAddress), alignof(GInetSocketAddress));
    printf("%s;%zu;%zu\n", "GInetSocketAddressClass", sizeof(GInetSocketAddressClass), alignof(GInetSocketAddressClass));
    printf("%s;%zu;%zu\n", "GInitableIface", sizeof(GInitableIface), alignof(GInitableIface));
    printf("%s;%zu;%zu\n", "GInputMessage", sizeof(GInputMessage), alignof(GInputMessage));
    printf("%s;%zu;%zu\n", "GInputStream", sizeof(GInputStream), alignof(GInputStream));
    printf("%s;%zu;%zu\n", "GInputStreamClass", sizeof(GInputStreamClass), alignof(GInputStreamClass));
    printf("%s;%zu;%zu\n", "GInputVector", sizeof(GInputVector), alignof(GInputVector));
    printf("%s;%zu;%zu\n", "GListModelInterface", sizeof(GListModelInterface), alignof(GListModelInterface));
    printf("%s;%zu;%zu\n", "GListStoreClass", sizeof(GListStoreClass), alignof(GListStoreClass));
    printf("%s;%zu;%zu\n", "GLoadableIconIface", sizeof(GLoadableIconIface), alignof(GLoadableIconIface));
    printf("%s;%zu;%zu\n", "GMemoryInputStream", sizeof(GMemoryInputStream), alignof(GMemoryInputStream));
    printf("%s;%zu;%zu\n", "GMemoryInputStreamClass", sizeof(GMemoryInputStreamClass), alignof(GMemoryInputStreamClass));
    printf("%s;%zu;%zu\n", "GMemoryMonitorInterface", sizeof(GMemoryMonitorInterface), alignof(GMemoryMonitorInterface));
    printf("%s;%zu;%zu\n", "GMemoryMonitorWarningLevel", sizeof(GMemoryMonitorWarningLevel), alignof(GMemoryMonitorWarningLevel));
    printf("%s;%zu;%zu\n", "GMemoryOutputStream", sizeof(GMemoryOutputStream), alignof(GMemoryOutputStream));
    printf("%s;%zu;%zu\n", "GMemoryOutputStreamClass", sizeof(GMemoryOutputStreamClass), alignof(GMemoryOutputStreamClass));
    printf("%s;%zu;%zu\n", "GMenuAttributeIter", sizeof(GMenuAttributeIter), alignof(GMenuAttributeIter));
    printf("%s;%zu;%zu\n", "GMenuAttributeIterClass", sizeof(GMenuAttributeIterClass), alignof(GMenuAttributeIterClass));
    printf("%s;%zu;%zu\n", "GMenuLinkIter", sizeof(GMenuLinkIter), alignof(GMenuLinkIter));
    printf("%s;%zu;%zu\n", "GMenuLinkIterClass", sizeof(GMenuLinkIterClass), alignof(GMenuLinkIterClass));
    printf("%s;%zu;%zu\n", "GMenuModel", sizeof(GMenuModel), alignof(GMenuModel));
    printf("%s;%zu;%zu\n", "GMenuModelClass", sizeof(GMenuModelClass), alignof(GMenuModelClass));
    printf("%s;%zu;%zu\n", "GMountIface", sizeof(GMountIface), alignof(GMountIface));
    printf("%s;%zu;%zu\n", "GMountMountFlags", sizeof(GMountMountFlags), alignof(GMountMountFlags));
    printf("%s;%zu;%zu\n", "GMountOperation", sizeof(GMountOperation), alignof(GMountOperation));
    printf("%s;%zu;%zu\n", "GMountOperationClass", sizeof(GMountOperationClass), alignof(GMountOperationClass));
    printf("%s;%zu;%zu\n", "GMountOperationResult", sizeof(GMountOperationResult), alignof(GMountOperationResult));
    printf("%s;%zu;%zu\n", "GMountUnmountFlags", sizeof(GMountUnmountFlags), alignof(GMountUnmountFlags));
    printf("%s;%zu;%zu\n", "GNativeSocketAddress", sizeof(GNativeSocketAddress), alignof(GNativeSocketAddress));
    printf("%s;%zu;%zu\n", "GNativeSocketAddressClass", sizeof(GNativeSocketAddressClass), alignof(GNativeSocketAddressClass));
    printf("%s;%zu;%zu\n", "GNativeVolumeMonitor", sizeof(GNativeVolumeMonitor), alignof(GNativeVolumeMonitor));
    printf("%s;%zu;%zu\n", "GNativeVolumeMonitorClass", sizeof(GNativeVolumeMonitorClass), alignof(GNativeVolumeMonitorClass));
    printf("%s;%zu;%zu\n", "GNetworkAddress", sizeof(GNetworkAddress), alignof(GNetworkAddress));
    printf("%s;%zu;%zu\n", "GNetworkAddressClass", sizeof(GNetworkAddressClass), alignof(GNetworkAddressClass));
    printf("%s;%zu;%zu\n", "GNetworkConnectivity", sizeof(GNetworkConnectivity), alignof(GNetworkConnectivity));
    printf("%s;%zu;%zu\n", "GNetworkMonitorInterface", sizeof(GNetworkMonitorInterface), alignof(GNetworkMonitorInterface));
    printf("%s;%zu;%zu\n", "GNetworkService", sizeof(GNetworkService), alignof(GNetworkService));
    printf("%s;%zu;%zu\n", "GNetworkServiceClass", sizeof(GNetworkServiceClass), alignof(GNetworkServiceClass));
    printf("%s;%zu;%zu\n", "GNotificationPriority", sizeof(GNotificationPriority), alignof(GNotificationPriority));
    printf("%s;%zu;%zu\n", "GOutputMessage", sizeof(GOutputMessage), alignof(GOutputMessage));
    printf("%s;%zu;%zu\n", "GOutputStream", sizeof(GOutputStream), alignof(GOutputStream));
    printf("%s;%zu;%zu\n", "GOutputStreamClass", sizeof(GOutputStreamClass), alignof(GOutputStreamClass));
    printf("%s;%zu;%zu\n", "GOutputStreamSpliceFlags", sizeof(GOutputStreamSpliceFlags), alignof(GOutputStreamSpliceFlags));
    printf("%s;%zu;%zu\n", "GOutputVector", sizeof(GOutputVector), alignof(GOutputVector));
    printf("%s;%zu;%zu\n", "GPasswordSave", sizeof(GPasswordSave), alignof(GPasswordSave));
    printf("%s;%zu;%zu\n", "GPermission", sizeof(GPermission), alignof(GPermission));
    printf("%s;%zu;%zu\n", "GPermissionClass", sizeof(GPermissionClass), alignof(GPermissionClass));
    printf("%s;%zu;%zu\n", "GPollableInputStreamInterface", sizeof(GPollableInputStreamInterface), alignof(GPollableInputStreamInterface));
    printf("%s;%zu;%zu\n", "GPollableOutputStreamInterface", sizeof(GPollableOutputStreamInterface), alignof(GPollableOutputStreamInterface));
    printf("%s;%zu;%zu\n", "GPollableReturn", sizeof(GPollableReturn), alignof(GPollableReturn));
    printf("%s;%zu;%zu\n", "GPowerProfileMonitorInterface", sizeof(GPowerProfileMonitorInterface), alignof(GPowerProfileMonitorInterface));
    printf("%s;%zu;%zu\n", "GProxyAddress", sizeof(GProxyAddress), alignof(GProxyAddress));
    printf("%s;%zu;%zu\n", "GProxyAddressClass", sizeof(GProxyAddressClass), alignof(GProxyAddressClass));
    printf("%s;%zu;%zu\n", "GProxyAddressEnumerator", sizeof(GProxyAddressEnumerator), alignof(GProxyAddressEnumerator));
    printf("%s;%zu;%zu\n", "GProxyAddressEnumeratorClass", sizeof(GProxyAddressEnumeratorClass), alignof(GProxyAddressEnumeratorClass));
    printf("%s;%zu;%zu\n", "GProxyInterface", sizeof(GProxyInterface), alignof(GProxyInterface));
    printf("%s;%zu;%zu\n", "GProxyResolverInterface", sizeof(GProxyResolverInterface), alignof(GProxyResolverInterface));
    printf("%s;%zu;%zu\n", "GRemoteActionGroupInterface", sizeof(GRemoteActionGroupInterface), alignof(GRemoteActionGroupInterface));
    printf("%s;%zu;%zu\n", "GResolver", sizeof(GResolver), alignof(GResolver));
    printf("%s;%zu;%zu\n", "GResolverClass", sizeof(GResolverClass), alignof(GResolverClass));
    printf("%s;%zu;%zu\n", "GResolverError", sizeof(GResolverError), alignof(GResolverError));
    printf("%s;%zu;%zu\n", "GResolverNameLookupFlags", sizeof(GResolverNameLookupFlags), alignof(GResolverNameLookupFlags));
    printf("%s;%zu;%zu\n", "GResolverRecordType", sizeof(GResolverRecordType), alignof(GResolverRecordType));
    printf("%s;%zu;%zu\n", "GResourceError", sizeof(GResourceError), alignof(GResourceError));
    printf("%s;%zu;%zu\n", "GResourceFlags", sizeof(GResourceFlags), alignof(GResourceFlags));
    printf("%s;%zu;%zu\n", "GResourceLookupFlags", sizeof(GResourceLookupFlags), alignof(GResourceLookupFlags));
    printf("%s;%zu;%zu\n", "GSeekableIface", sizeof(GSeekableIface), alignof(GSeekableIface));
    printf("%s;%zu;%zu\n", "GSettings", sizeof(GSettings), alignof(GSettings));
    printf("%s;%zu;%zu\n", "GSettingsBackend", sizeof(GSettingsBackend), alignof(GSettingsBackend));
    printf("%s;%zu;%zu\n", "GSettingsBackendClass", sizeof(GSettingsBackendClass), alignof(GSettingsBackendClass));
    printf("%s;%zu;%zu\n", "GSettingsBindFlags", sizeof(GSettingsBindFlags), alignof(GSettingsBindFlags));
    printf("%s;%zu;%zu\n", "GSettingsClass", sizeof(GSettingsClass), alignof(GSettingsClass));
    printf("%s;%zu;%zu\n", "GSimpleActionGroup", sizeof(GSimpleActionGroup), alignof(GSimpleActionGroup));
    printf("%s;%zu;%zu\n", "GSimpleActionGroupClass", sizeof(GSimpleActionGroupClass), alignof(GSimpleActionGroupClass));
    printf("%s;%zu;%zu\n", "GSimpleProxyResolver", sizeof(GSimpleProxyResolver), alignof(GSimpleProxyResolver));
    printf("%s;%zu;%zu\n", "GSimpleProxyResolverClass", sizeof(GSimpleProxyResolverClass), alignof(GSimpleProxyResolverClass));
    printf("%s;%zu;%zu\n", "GSocket", sizeof(GSocket), alignof(GSocket));
    printf("%s;%zu;%zu\n", "GSocketAddress", sizeof(GSocketAddress), alignof(GSocketAddress));
    printf("%s;%zu;%zu\n", "GSocketAddressClass", sizeof(GSocketAddressClass), alignof(GSocketAddressClass));
    printf("%s;%zu;%zu\n", "GSocketAddressEnumerator", sizeof(GSocketAddressEnumerator), alignof(GSocketAddressEnumerator));
    printf("%s;%zu;%zu\n", "GSocketAddressEnumeratorClass", sizeof(GSocketAddressEnumeratorClass), alignof(GSocketAddressEnumeratorClass));
    printf("%s;%zu;%zu\n", "GSocketClass", sizeof(GSocketClass), alignof(GSocketClass));
    printf("%s;%zu;%zu\n", "GSocketClient", sizeof(GSocketClient), alignof(GSocketClient));
    printf("%s;%zu;%zu\n", "GSocketClientClass", sizeof(GSocketClientClass), alignof(GSocketClientClass));
    printf("%s;%zu;%zu\n", "GSocketClientEvent", sizeof(GSocketClientEvent), alignof(GSocketClientEvent));
    printf("%s;%zu;%zu\n", "GSocketConnectableIface", sizeof(GSocketConnectableIface), alignof(GSocketConnectableIface));
    printf("%s;%zu;%zu\n", "GSocketConnection", sizeof(GSocketConnection), alignof(GSocketConnection));
    printf("%s;%zu;%zu\n", "GSocketConnectionClass", sizeof(GSocketConnectionClass), alignof(GSocketConnectionClass));
    printf("%s;%zu;%zu\n", "GSocketControlMessage", sizeof(GSocketControlMessage), alignof(GSocketControlMessage));
    printf("%s;%zu;%zu\n", "GSocketControlMessageClass", sizeof(GSocketControlMessageClass), alignof(GSocketControlMessageClass));
    printf("%s;%zu;%zu\n", "GSocketFamily", sizeof(GSocketFamily), alignof(GSocketFamily));
    printf("%s;%zu;%zu\n", "GSocketListener", sizeof(GSocketListener), alignof(GSocketListener));
    printf("%s;%zu;%zu\n", "GSocketListenerClass", sizeof(GSocketListenerClass), alignof(GSocketListenerClass));
    printf("%s;%zu;%zu\n", "GSocketListenerEvent", sizeof(GSocketListenerEvent), alignof(GSocketListenerEvent));
    printf("%s;%zu;%zu\n", "GSocketMsgFlags", sizeof(GSocketMsgFlags), alignof(GSocketMsgFlags));
    printf("%s;%zu;%zu\n", "GSocketProtocol", sizeof(GSocketProtocol), alignof(GSocketProtocol));
    printf("%s;%zu;%zu\n", "GSocketService", sizeof(GSocketService), alignof(GSocketService));
    printf("%s;%zu;%zu\n", "GSocketServiceClass", sizeof(GSocketServiceClass), alignof(GSocketServiceClass));
    printf("%s;%zu;%zu\n", "GSocketType", sizeof(GSocketType), alignof(GSocketType));
    printf("%s;%zu;%zu\n", "GStaticResource", sizeof(GStaticResource), alignof(GStaticResource));
    printf("%s;%zu;%zu\n", "GSubprocessFlags", sizeof(GSubprocessFlags), alignof(GSubprocessFlags));
    printf("%s;%zu;%zu\n", "GTcpConnection", sizeof(GTcpConnection), alignof(GTcpConnection));
    printf("%s;%zu;%zu\n", "GTcpConnectionClass", sizeof(GTcpConnectionClass), alignof(GTcpConnectionClass));
    printf("%s;%zu;%zu\n", "GTcpWrapperConnection", sizeof(GTcpWrapperConnection), alignof(GTcpWrapperConnection));
    printf("%s;%zu;%zu\n", "GTcpWrapperConnectionClass", sizeof(GTcpWrapperConnectionClass), alignof(GTcpWrapperConnectionClass));
    printf("%s;%zu;%zu\n", "GTestDBusFlags", sizeof(GTestDBusFlags), alignof(GTestDBusFlags));
    printf("%s;%zu;%zu\n", "GThreadedSocketService", sizeof(GThreadedSocketService), alignof(GThreadedSocketService));
    printf("%s;%zu;%zu\n", "GThreadedSocketServiceClass", sizeof(GThreadedSocketServiceClass), alignof(GThreadedSocketServiceClass));
    printf("%s;%zu;%zu\n", "GTlsAuthenticationMode", sizeof(GTlsAuthenticationMode), alignof(GTlsAuthenticationMode));
    printf("%s;%zu;%zu\n", "GTlsBackendInterface", sizeof(GTlsBackendInterface), alignof(GTlsBackendInterface));
    printf("%s;%zu;%zu\n", "GTlsCertificate", sizeof(GTlsCertificate), alignof(GTlsCertificate));
    printf("%s;%zu;%zu\n", "GTlsCertificateClass", sizeof(GTlsCertificateClass), alignof(GTlsCertificateClass));
    printf("%s;%zu;%zu\n", "GTlsCertificateFlags", sizeof(GTlsCertificateFlags), alignof(GTlsCertificateFlags));
    printf("%s;%zu;%zu\n", "GTlsCertificateRequestFlags", sizeof(GTlsCertificateRequestFlags), alignof(GTlsCertificateRequestFlags));
    printf("%s;%zu;%zu\n", "GTlsChannelBindingError", sizeof(GTlsChannelBindingError), alignof(GTlsChannelBindingError));
    printf("%s;%zu;%zu\n", "GTlsChannelBindingType", sizeof(GTlsChannelBindingType), alignof(GTlsChannelBindingType));
    printf("%s;%zu;%zu\n", "GTlsClientConnectionInterface", sizeof(GTlsClientConnectionInterface), alignof(GTlsClientConnectionInterface));
    printf("%s;%zu;%zu\n", "GTlsConnection", sizeof(GTlsConnection), alignof(GTlsConnection));
    printf("%s;%zu;%zu\n", "GTlsConnectionClass", sizeof(GTlsConnectionClass), alignof(GTlsConnectionClass));
    printf("%s;%zu;%zu\n", "GTlsDatabase", sizeof(GTlsDatabase), alignof(GTlsDatabase));
    printf("%s;%zu;%zu\n", "GTlsDatabaseClass", sizeof(GTlsDatabaseClass), alignof(GTlsDatabaseClass));
    printf("%s;%zu;%zu\n", "GTlsDatabaseLookupFlags", sizeof(GTlsDatabaseLookupFlags), alignof(GTlsDatabaseLookupFlags));
    printf("%s;%zu;%zu\n", "GTlsDatabaseVerifyFlags", sizeof(GTlsDatabaseVerifyFlags), alignof(GTlsDatabaseVerifyFlags));
    printf("%s;%zu;%zu\n", "GTlsError", sizeof(GTlsError), alignof(GTlsError));
    printf("%s;%zu;%zu\n", "GTlsFileDatabaseInterface", sizeof(GTlsFileDatabaseInterface), alignof(GTlsFileDatabaseInterface));
    printf("%s;%zu;%zu\n", "GTlsInteraction", sizeof(GTlsInteraction), alignof(GTlsInteraction));
    printf("%s;%zu;%zu\n", "GTlsInteractionClass", sizeof(GTlsInteractionClass), alignof(GTlsInteractionClass));
    printf("%s;%zu;%zu\n", "GTlsInteractionResult", sizeof(GTlsInteractionResult), alignof(GTlsInteractionResult));
    printf("%s;%zu;%zu\n", "GTlsPassword", sizeof(GTlsPassword), alignof(GTlsPassword));
    printf("%s;%zu;%zu\n", "GTlsPasswordClass", sizeof(GTlsPasswordClass), alignof(GTlsPasswordClass));
    printf("%s;%zu;%zu\n", "GTlsPasswordFlags", sizeof(GTlsPasswordFlags), alignof(GTlsPasswordFlags));
    printf("%s;%zu;%zu\n", "GTlsProtocolVersion", sizeof(GTlsProtocolVersion), alignof(GTlsProtocolVersion));
    printf("%s;%zu;%zu\n", "GTlsRehandshakeMode", sizeof(GTlsRehandshakeMode), alignof(GTlsRehandshakeMode));
    printf("%s;%zu;%zu\n", "GTlsServerConnectionInterface", sizeof(GTlsServerConnectionInterface), alignof(GTlsServerConnectionInterface));
    printf("%s;%zu;%zu\n", "GUnixConnection", sizeof(GUnixConnection), alignof(GUnixConnection));
    printf("%s;%zu;%zu\n", "GUnixConnectionClass", sizeof(GUnixConnectionClass), alignof(GUnixConnectionClass));
    printf("%s;%zu;%zu\n", "GUnixCredentialsMessage", sizeof(GUnixCredentialsMessage), alignof(GUnixCredentialsMessage));
    printf("%s;%zu;%zu\n", "GUnixCredentialsMessageClass", sizeof(GUnixCredentialsMessageClass), alignof(GUnixCredentialsMessageClass));
    printf("%s;%zu;%zu\n", "GUnixFDList", sizeof(GUnixFDList), alignof(GUnixFDList));
    printf("%s;%zu;%zu\n", "GUnixFDListClass", sizeof(GUnixFDListClass), alignof(GUnixFDListClass));
    printf("%s;%zu;%zu\n", "GUnixFDMessage", sizeof(GUnixFDMessage), alignof(GUnixFDMessage));
    printf("%s;%zu;%zu\n", "GUnixFDMessageClass", sizeof(GUnixFDMessageClass), alignof(GUnixFDMessageClass));
    printf("%s;%zu;%zu\n", "GUnixInputStream", sizeof(GUnixInputStream), alignof(GUnixInputStream));
    printf("%s;%zu;%zu\n", "GUnixInputStreamClass", sizeof(GUnixInputStreamClass), alignof(GUnixInputStreamClass));
    printf("%s;%zu;%zu\n", "GUnixOutputStream", sizeof(GUnixOutputStream), alignof(GUnixOutputStream));
    printf("%s;%zu;%zu\n", "GUnixOutputStreamClass", sizeof(GUnixOutputStreamClass), alignof(GUnixOutputStreamClass));
    printf("%s;%zu;%zu\n", "GUnixSocketAddress", sizeof(GUnixSocketAddress), alignof(GUnixSocketAddress));
    printf("%s;%zu;%zu\n", "GUnixSocketAddressClass", sizeof(GUnixSocketAddressClass), alignof(GUnixSocketAddressClass));
    printf("%s;%zu;%zu\n", "GUnixSocketAddressType", sizeof(GUnixSocketAddressType), alignof(GUnixSocketAddressType));
    printf("%s;%zu;%zu\n", "GVfs", sizeof(GVfs), alignof(GVfs));
    printf("%s;%zu;%zu\n", "GVfsClass", sizeof(GVfsClass), alignof(GVfsClass));
    printf("%s;%zu;%zu\n", "GVolumeIface", sizeof(GVolumeIface), alignof(GVolumeIface));
    printf("%s;%zu;%zu\n", "GVolumeMonitor", sizeof(GVolumeMonitor), alignof(GVolumeMonitor));
    printf("%s;%zu;%zu\n", "GVolumeMonitorClass", sizeof(GVolumeMonitorClass), alignof(GVolumeMonitorClass));
    printf("%s;%zu;%zu\n", "GZlibCompressorClass", sizeof(GZlibCompressorClass), alignof(GZlibCompressorClass));
    printf("%s;%zu;%zu\n", "GZlibCompressorFormat", sizeof(GZlibCompressorFormat), alignof(GZlibCompressorFormat));
    printf("%s;%zu;%zu\n", "GZlibDecompressorClass", sizeof(GZlibDecompressorClass), alignof(GZlibDecompressorClass));
    return 0;
}
