// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, GStr};

#[doc(alias = "G_CSET_A_2_Z")]
pub static CSET_A_2_Z: &GStr = unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_CSET_A_2_Z) };
#[doc(alias = "G_CSET_DIGITS")]
pub static CSET_DIGITS: &GStr = unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_CSET_DIGITS) };
#[doc(alias = "G_CSET_a_2_z")]
pub static CSET_a_2_z: &GStr = unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_CSET_a_2_z) };
#[doc(alias = "G_KEY_FILE_DESKTOP_GROUP")]
pub static KEY_FILE_DESKTOP_GROUP: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_GROUP) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_ACTIONS")]
pub static KEY_FILE_DESKTOP_KEY_ACTIONS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_ACTIONS) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_CATEGORIES")]
pub static KEY_FILE_DESKTOP_KEY_CATEGORIES: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_CATEGORIES) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_COMMENT")]
pub static KEY_FILE_DESKTOP_KEY_COMMENT: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_COMMENT) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_DBUS_ACTIVATABLE")]
pub static KEY_FILE_DESKTOP_KEY_DBUS_ACTIVATABLE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_DBUS_ACTIVATABLE) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_EXEC")]
pub static KEY_FILE_DESKTOP_KEY_EXEC: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_EXEC) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_GENERIC_NAME")]
pub static KEY_FILE_DESKTOP_KEY_GENERIC_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_GENERIC_NAME) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_HIDDEN")]
pub static KEY_FILE_DESKTOP_KEY_HIDDEN: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_HIDDEN) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_ICON")]
pub static KEY_FILE_DESKTOP_KEY_ICON: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_ICON) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_MIME_TYPE")]
pub static KEY_FILE_DESKTOP_KEY_MIME_TYPE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_MIME_TYPE) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_NAME")]
pub static KEY_FILE_DESKTOP_KEY_NAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_NAME) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_NOT_SHOW_IN")]
pub static KEY_FILE_DESKTOP_KEY_NOT_SHOW_IN: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_NOT_SHOW_IN) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_NO_DISPLAY")]
pub static KEY_FILE_DESKTOP_KEY_NO_DISPLAY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_NO_DISPLAY) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_ONLY_SHOW_IN")]
pub static KEY_FILE_DESKTOP_KEY_ONLY_SHOW_IN: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_ONLY_SHOW_IN) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_PATH")]
pub static KEY_FILE_DESKTOP_KEY_PATH: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_PATH) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_STARTUP_NOTIFY")]
pub static KEY_FILE_DESKTOP_KEY_STARTUP_NOTIFY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_STARTUP_NOTIFY) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_STARTUP_WM_CLASS")]
pub static KEY_FILE_DESKTOP_KEY_STARTUP_WM_CLASS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_STARTUP_WM_CLASS) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_TERMINAL")]
pub static KEY_FILE_DESKTOP_KEY_TERMINAL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_TERMINAL) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_TRY_EXEC")]
pub static KEY_FILE_DESKTOP_KEY_TRY_EXEC: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_TRY_EXEC) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_TYPE")]
pub static KEY_FILE_DESKTOP_KEY_TYPE: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_TYPE) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_URL")]
pub static KEY_FILE_DESKTOP_KEY_URL: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_URL) };
#[doc(alias = "G_KEY_FILE_DESKTOP_KEY_VERSION")]
pub static KEY_FILE_DESKTOP_KEY_VERSION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_KEY_VERSION) };
#[doc(alias = "G_KEY_FILE_DESKTOP_TYPE_APPLICATION")]
pub static KEY_FILE_DESKTOP_TYPE_APPLICATION: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_TYPE_APPLICATION) };
#[doc(alias = "G_KEY_FILE_DESKTOP_TYPE_DIRECTORY")]
pub static KEY_FILE_DESKTOP_TYPE_DIRECTORY: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_TYPE_DIRECTORY) };
#[doc(alias = "G_KEY_FILE_DESKTOP_TYPE_LINK")]
pub static KEY_FILE_DESKTOP_TYPE_LINK: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_KEY_FILE_DESKTOP_TYPE_LINK) };
#[doc(alias = "G_OPTION_REMAINING")]
pub static OPTION_REMAINING: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_OPTION_REMAINING) };
#[doc(alias = "G_STR_DELIMITERS")]
pub static STR_DELIMITERS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_STR_DELIMITERS) };
#[cfg(feature = "v2_60")]
#[cfg_attr(docsrs, doc(cfg(feature = "v2_60")))]
#[doc(alias = "G_TEST_OPTION_ISOLATE_DIRS")]
pub static TEST_OPTION_ISOLATE_DIRS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_TEST_OPTION_ISOLATE_DIRS) };
#[cfg(feature = "v2_84")]
#[cfg_attr(docsrs, doc(cfg(feature = "v2_84")))]
#[doc(alias = "G_TEST_OPTION_NONFATAL_ASSERTIONS")]
pub static TEST_OPTION_NONFATAL_ASSERTIONS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_TEST_OPTION_NONFATAL_ASSERTIONS) };
#[cfg(feature = "v2_84")]
#[cfg_attr(docsrs, doc(cfg(feature = "v2_84")))]
#[doc(alias = "G_TEST_OPTION_NO_PRGNAME")]
pub static TEST_OPTION_NO_PRGNAME: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_TEST_OPTION_NO_PRGNAME) };
#[doc(alias = "G_URI_RESERVED_CHARS_GENERIC_DELIMITERS")]
pub static URI_RESERVED_CHARS_GENERIC_DELIMITERS: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::G_URI_RESERVED_CHARS_GENERIC_DELIMITERS) };
#[doc(alias = "G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS")]
pub static URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS: &GStr = unsafe {
    GStr::from_utf8_with_nul_unchecked(ffi::G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS)
};
