// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, ShortcutAction};

glib::wrapper! {
    #[doc(alias = "GtkCallbackAction")]
    pub struct CallbackAction(Object<ffi::GtkCallbackAction, ffi::GtkCallbackActionClass>) @extends ShortcutAction;

    match fn {
        type_ => || ffi::gtk_callback_action_get_type(),
    }
}

impl CallbackAction {}
